#include "gview.h"

/*
 * GViewApp.c contains all of the functions used to manipulate 
 * the gtk widget portions of the application 
 */

/* allocate space for the GViewApp and initialize its widgets */
GViewApp *
gview_new_app (GViewImage *img)
{
  GViewApp *ret_app;
  gchar window_title[1024];

  sprintf (window_title, "GView: %s", utils_get_filename());

  /* allocate space for the struct */
  ret_app = (GViewApp *)malloc (sizeof (GViewApp));

  /* first a GtkWindow must be constructed as a top level */
  /* container for all additional widgets */
  ret_app->gtk_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  /* store the GViewImage */
  ret_app->gview_img = img;

  /* GdkPixbufs are great for manipulating images */
  /* but to be displayed in a gtk app the GdkPixbuf must */
  /* be converted to a GtkImage widget */
  ret_app->gtk_img = gtk_image_new_from_pixbuf (image_get_pixbuf (img));
  /* add the GtkImage as a child, to the GtkWindow */
  gtk_container_add (GTK_CONTAINER (ret_app->gtk_window), ret_app->gtk_img);

  /* set the application window's title */
  gtk_window_set_title (GTK_WINDOW (ret_app->gtk_window), window_title);

  /* now connect functions to window events such as mouse clicks */
  /* using g_signal_connect instruct GTK to execute functions when the */
  /* GtkWindow receives specified events */
  /* 'delete' and 'destroy' are events that occur when a window's */
  /* close button is clicked */
  g_signal_connect (G_OBJECT (ret_app->gtk_window), "delete_event",
  			G_CALLBACK (gview_stop), NULL);
  g_signal_connect (G_OBJECT (ret_app->gtk_window), "destroy",
  			G_CALLBACK (gview_stop), NULL);
  /* execute 'dispatch_keypress' when a keyboard button is pressed */
  g_signal_connect (G_OBJECT (ret_app->gtk_window), "key_press_event",
  			G_CALLBACK (dispatch_keypress), ret_app);
  /* execute 'dispatch_scroll' when the mouse wheel is scrolled */
  g_signal_connect (G_OBJECT (ret_app->gtk_window), "scroll_event",
  			G_CALLBACK (dispatch_scroll), ret_app);
  /* execute 'dispatch_button_press' when a mouse button is pressed */
  g_signal_connect (G_OBJECT (ret_app->gtk_window), "button_press_event",
  			G_CALLBACK (dispatch_button_press), ret_app);

  /* now specific events have been associated with appropriate functions */
  /* now we must inform gtk what type of events to watch for regarding the GtkWindw */
  gtk_widget_set_events (ret_app->gtk_window, GDK_KEY_PRESS_MASK | GDK_SCROLL_MASK);

  return ret_app;
}

void
gview_show_app (GViewApp *app)
{
  /* gtk_widget_show_all will recursively display each child widget */
  /* of the GtkWindow */
  gtk_widget_show_all (app->gtk_window);
}

/* gview_stop is called when a 'destroy' or 'delete' event occurs */
static void
gview_stop (GtkWidget *widget, gpointer data)
{
  /* calling gtk_main_quit forces the 'gtk_main' function */
  /* called in the 'main' function to stop iterating */
  gtk_main_quit ();
}

/* re-set the GtkImage child of the GtkWindow */
void
gview_set_image (GViewApp *app, GdkPixbuf *img)
{
  gint width, height;

  /* get the dimensions of the image */
  image_get_size (app->gview_img, &width, &height);
  /* set the GtkImage to display the GdkPixbuf */
  gtk_image_set_from_pixbuf (GTK_IMAGE(app->gtk_img), img);
  /* resize the window to fit the new image */
  /* particularly necessary when shrinking the image */
  gtk_window_resize (GTK_WINDOW (app->gtk_window), width, height);
}

/* display a small dialog to display information about the image */
/* such as size in pixels... */
static void
disp_img_info (GViewApp *app)
{
	/* a dialog, a GtkDialog, is a specialized type of GtkWindow */
	GtkWidget *dialog;
	gint origWidth, origHeight;
	gint width, height;
	
	/* retrieve the image's information */
	image_get_orig_size (app->gview_img, &origWidth, &origHeight);
	image_get_size (app->gview_img, &width, &height);
	
	/* create a GtkDialog */
	/* a GtkDialog of type 'MESSAGE_INFO' with but one button; a Close button */
	dialog = gtk_message_dialog_new (GTK_WINDOW(app->gtk_window), 
										GTK_DIALOG_DESTROY_WITH_PARENT, 
										GTK_MESSAGE_INFO, 
										GTK_BUTTONS_CLOSE, 
										"Filename: %s\nOrig. Width: %dpx\nOrig. Height: %dpx\nWidth: %dpx\nHeight: %dpx",
										utils_get_filename (),
										origWidth, origHeight,
										width, height);
	
	/* just like with the GtkWindow functions must be associated */
	/* with events.  When the Close button is clicked it must close */
	/* the dialog */
	g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
				G_CALLBACK (gtk_widget_destroy),
				GTK_OBJECT (dialog));
	
	/* show the GtkDialog; it will run (display) until 'Close' is clicked */
	gtk_widget_show (dialog);	
}

/* each event emits an Event structure.  This structure contains information */
/* such as what key was pressed or what button was pressed and where */
static void
dispatch_keypress (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
  /* examine the key that was pressed */
  switch (event->keyval){
	/* use GDK definitions to compare the keyval against known key values */
    case GDK_Up:
		/* increase the size of the displayed image */
		gview_set_image ((GViewApp *)data, image_resize (((GViewApp *)data)->gview_img, 1.25));
    	break;
    case GDK_Down:
		/* decrease the size of the displayed image */
		gview_set_image ((GViewApp *)data, image_resize (((GViewApp *)data)->gview_img, 0.25));
    	break;
	case GDK_f:
		/* display the original 'unscaled' version of the image */
		gview_set_image ((GViewApp *)data, image_get_orig_pixbuf (((GViewApp *)data)->gview_img));
		break;
    case GDK_i:
		/* display the images information in a dialog */
		disp_img_info ((GViewApp *)data);
		break;
    case GDK_r:
		/* restore the image back to its original 'scaled' size */
		gview_set_image ((GViewApp *)data, image_get_orig_scaled_pixbuf(((GViewApp *)data)->gview_img));
		break;
    case GDK_x:
		/* quit the application */
		gview_stop (widget, NULL);
		break;
    default:
		break;
  }
}

static void
dispatch_scroll (GtkWidget *widget, GdkEvent *event, gpointer data)
{
  /* determine if the scroll wheel was rolled up or down */
  switch (((GdkEventScroll*)event)->direction){
    case GDK_SCROLL_UP:
		/* increase the size of the displayed image */
		gview_set_image ((GViewApp *)data, image_resize (((GViewApp *)data)->gview_img, 1.25));
		break;
    case GDK_SCROLL_DOWN:
		/* decrease the size of the displayed image */
		gview_set_image ((GViewApp *)data, image_resize (((GViewApp *)data)->gview_img, 0.25));
		break;
  }
}

static void 
dispatch_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
  /* figure out what mouse button was pressed */
  switch (((GdkEventButton*)event)->button){
    case LEFT_MOUSE:
		/* to allow the user to drag the window without having to use */
	    /* a window manager control, allow the entire window to be dragged */
	    /* by clicking (and holding) anywhere within the window */
		gtk_window_begin_move_drag (GTK_WINDOW (widget),
									((GdkEventButton *)event)->button,
									((GdkEventButton *)event)->x_root,
									((GdkEventButton *)event)->y_root,
									((GdkEventButton *)event)->time);
		break;
    case MIDDLE_MOUSE:
		/* zoom in on the image */
		gview_set_image (data, image_zoom (((GViewApp *)data)->gview_img, ((GdkEventButton *)event)->x, ((GdkEventButton *)event)->y));
		break;
    case RIGHT_MOUSE:
		/* restore the image back to its original scaled proportions */
		gview_set_image (data, image_get_pixbuf(((GViewApp *)data)->gview_img));
		break;
  }
}
