#include "gviewimage.h"

/* 'image_fit_to_screen' ensures that a displayed image's */
/* dimensions do not exceed the capacity of the desktop */
GViewImage *
image_fit_to_screen (GViewImage *img)
{
  GdkScreen *screen; /* the struct representing the desktop */
  gint screen_height, screen_width;
  GdkPixbuf *tmp_pixbuf; /* a buffer pixbuf if you will */
  gint tmp_gdk_height, tmp_gdk_width;
  /* tmp_gdk_height and tmp_gdk_width must be initialized */
  /* so as to prevent them from initializing as any crazy value */
  /* like -31125939 */
  tmp_gdk_height = tmp_gdk_width = 0;
	
  /* get a handle to the screen for its measurements */
  screen = gdk_screen_get_default ();
  /* using the gdk library functions, get the height and width */
  screen_height = gdk_screen_get_height (screen);
  screen_width = gdk_screen_get_width (screen);
  /* leave some buffer room around our image window */
  /* allowing room for things like taskbars that may be on the screen */
  screen_height -= SCREEN_MARGIN;
  screen_width -= SCREEN_MARGIN;
	
  /* check that the image is not bigger than the screen */
  /* if the image is taller than the desktop then reset */
  /* the tmp_gdk_height to reflect this */
  if (img->orig_height > screen_height)
    tmp_gdk_height = screen_height;

  if (img->orig_width > screen_width)
    tmp_gdk_width = screen_width;

  /* if image is too big, resize it */
  /* if either tmp variables were set then the image DOES exceed */
  /* the size of the desktop and we must resize it to fit */
  if (tmp_gdk_height || tmp_gdk_width){
	/* calculate the width to height ratio */
    float w_ratio = img->orig_width / screen_width;
    float h_ratio = img->orig_height / screen_height;

    /* calculate new gdkpixbuf measurements */
	/* determine if the image is wider than tall or vice versa */
	/* calculate a new height or width that keeps the image ratio */
    if (w_ratio >= h_ratio){
      tmp_gdk_height = ((float)img->orig_height / (float)img->orig_width) * tmp_gdk_width;
    }else{
      tmp_gdk_width = ((float)img->orig_width / (float)img->orig_height) * tmp_gdk_height;
    }

    /* create resized gdkpixbuf */
    img->disp_img = gdk_pixbuf_scale_simple (img->orig_img,
    					    tmp_gdk_width,
					    	tmp_gdk_height,
					    	GDK_INTERP_BILINEAR);

	/* remember to store the new height and width of the image */
    img->scaled_height = img->current_height = tmp_gdk_height;
    img->scaled_width = img->current_width = tmp_gdk_width;
  }
  
  return img;
}

/* allocate space for the image, initalize it and return it */
GViewImage *
image_new_image (char *img_filename)
{
  GViewImage *img;

  img = (GViewImage *)malloc (sizeof (GViewImage));

  /* create the image and record its measurements */
  /* the GdkPixbuf structure is used store the image data */
  img->orig_img=img->disp_img= gdk_pixbuf_new_from_file (img_filename, NULL);
  img->orig_height=img->scaled_height=img->current_height= gdk_pixbuf_get_height (img->orig_img);
  img->orig_width=img->scaled_width=img->current_width= gdk_pixbuf_get_width (img->orig_img);

  /* now with the image loaded, make sure it fits to the screen */
  image_fit_to_screen (img);
	
  return img;
}

/* some simple 'getter' functions to return members of the GViewImage struct */
GdkPixbuf *
image_get_pixbuf (GViewImage *img)
{
  return img->disp_img;
}

GdkPixbuf *
image_get_orig_scaled_pixbuf (GViewImage *img)
{
  image_fit_to_screen (img);
  return img->disp_img;
}

GdkPixbuf *
image_get_orig_pixbuf (GViewImage *img)
{
	return img->orig_img;
}

void
image_get_orig_size (GViewImage *img, gint *width, gint *height)
{
  *width = img->orig_width;
  *height = img->orig_height;
}

void
image_get_size (GViewImage *img, gint *width, gint *height)
{
  *width = img->current_width;
  *height = img->current_height;
}

/* 'image_resize' will allow the user to enlarge */
/* or shrink the displayed image */
GdkPixbuf *
image_resize (GViewImage *img, float r_factor)
{
  gint new_gdk_width, new_gdk_height;
  gint curr_width, curr_height;
  gint pos_x, pos_y;
	
  curr_width = img->current_width;
  curr_height = img->current_height;

  /* r_factor is a number which indicates by what factor */
  /* to increase or decrease the size of the image */
  if (r_factor < 1.0){
	/* calculate the new size; of course if r_factor is greater */
	/* than 1.0 (such as 1.25) then the image is increased to 125% */
	/* of its original size */
    new_gdk_width = (gint)(curr_width - (curr_width * r_factor));
    new_gdk_height = (gint)(curr_height - (curr_height * r_factor));
  }else{
    new_gdk_width = (gint)(curr_width * r_factor);
    new_gdk_height = (gint)(curr_height * r_factor);
  }

  /* set the size members of the GViewImage struct */
  img->current_width = img->scaled_width = new_gdk_width;
  img->current_height = img->scaled_height = new_gdk_height;

  /* store the newly scaled GdkPixbuf to the GViewImage struct */
  /* GViewImage has two GdkPixbuf structs; orig_img contains the original image */
  /* and disp_img stores a copy of the manipulated image */
  /* the original is kept around to preserve quality when scaled copies are made */
  img->disp_img = gdk_pixbuf_scale_simple (img->orig_img, new_gdk_width, new_gdk_height, GDK_INTERP_BILINEAR);

  return img->disp_img;
}

/* image_zoom allows the user to select an area and 'zoom' in on it */
GdkPixbuf *
image_zoom (GViewImage *img, gint x, gint y)
{
  gint half_zoom_height, half_zoom_width;
  gint zoom_x_start, zoom_y_start;
  gint zoom_height, zoom_width;
  GdkPixbuf *zoom_img;
 
  /* first we make some simple necessary calculations */
  half_zoom_height = img->current_height * ZOOM_PERCENT;
  half_zoom_width = img->current_width * ZOOM_PERCENT;
  zoom_height = 2*half_zoom_height;
  zoom_width = 2*half_zoom_width;
  zoom_x_start = x - half_zoom_width;
  zoom_y_start = y - half_zoom_height;
  
  /* if the image is wider than tall or vice versa */
  /* then we will need to adjust the number of pixels by which */
  /* to zoom so as to preserve the aspect ratio in regards to the */
  /* already sized gtk window */
  if (img->current_width > img->current_height)
	zoom_width = (img->current_width / img->current_height) * zoom_width;
	
  if (img->current_height > img->current_width)
	zoom_height = (img->current_height / img->current_width) * zoom_height;

  /* ensure that the calculated starting point of the zoomed image portion */
  /* does not exceed the boundaries of the existing displayed image */
  if (zoom_x_start < 0) zoom_x_start = 0;
  if (zoom_y_start < 0) zoom_y_start = 0;
  if ((zoom_x_start + zoom_width) > img->current_width)
    zoom_x_start = (img->current_width - zoom_width);
  if ((zoom_y_start + zoom_height) > img->current_height)
	zoom_y_start = (img->current_height - zoom_height);

  /* use the gdk libary again to generate a pixbuf which is a */
  /* 'sub-image' of the displayed image */
  img->disp_img = gdk_pixbuf_new_subpixbuf (img->disp_img,
				       zoom_x_start,
				       zoom_y_start,
				       zoom_width,
				       zoom_height);

  /* now scale the sub-image to fit the displayed gtk window */
  img->disp_img = gdk_pixbuf_scale_simple (img->disp_img,
  											img->current_width,
  											img->current_height,
  											GDK_INTERP_BILINEAR);

  /* cleanup; store the displayed images (the zoomed image) new dimensions */
  img->current_height = gdk_pixbuf_get_height (img->disp_img);
  img->current_width = gdk_pixbuf_get_width (img->disp_img);

  return img->disp_img;
}
